------------------------------------------------------------------------------
--	Caches all items currently equiped items and all items in the bags		--
------------------------------------------------------------------------------
local ItemCache =  FlexBar2:NewModule("ItemCache", "AceEvent-2.0");
-- If this var is incremented, SetDefaults() will be called for this module thus resetting all its settings. Though there are no settings for this module, it has to be declared
ItemCache.Version = 1;

local Cache = {};
ItemCache.Cache = Cache;

function ItemCache:OnInitialize()
	-- Build cache upon initial loading
    self:BuildItemCache();
	-- build cache when getting the BAG_UPDATE or UNIT_INVENTORY_CHANGED event for the player
    self:RegisterEvent("BAG_UPDATE", "BuildItemCache");
    self:RegisterEvent("UNIT_INVENTORY_CHANGED", function(UnitId) if(UnitId == "player") then ItemCache:BuildItemCache(); end end);
end

function ItemCache:BuildItemCache()
    FlexBar2:Debug("Building cache");
	-- Clear the old cache first
    for ItemLink, _ in pairs(Cache) do
        for Info, _ in pairs(Cache[ItemLink]) do
            Cache[ItemLink][Info] = nil;
        end
    end
	-- Loop through all bags
    for Bag = 0, 4 do
		-- loop through all slots in current bag
        for Slot =1, GetContainerNumSlots(Bag) do
			-- Get itemlink
            local ItemLink = GetContainerItemLink(Bag, Slot);
            if(ItemLink) then
				-- Put itemlink: in front of the itemlink :p
                ItemLink = "itemlink:" .. ItemLink
				-- Sanity check 
                if(type(Cache[ItemLink]) ~= "table") then Cache[ItemLink] = {}; end
				-- Save bag id & bagslot
                Cache[ItemLink].Bag = Bag;
                Cache[ItemLink].BagSlot = Slot;
            end
        end
    end
	-- Loop through all equiped slots
    for Slot = 0, 23 do
		-- Get itemlink
        local ItemLink = GetInventoryItemLink("player", Slot);
        if(ItemLink) then
			-- Put itemlink: in front of the itemlink :p
            ItemLink = "itemlink:" .. ItemLink;
			-- Sanity check
            if(type(Cache[ItemLink]) ~= "table") then Cache[ItemLink] = {}; end
			-- Save inventory slot
            Cache[ItemLink].InvSlot = Slot;
        end
    end
end

-- Small function to get something from the cache easely
function ItemCache:GetItemInfo(ItemLink)
    return Cache[ItemLink] or Cache["itemlink:" .. ItemLink] or false;
end
